local addon_name, addon = ...
local menu = addon.menu

local util = addon.utility
local tt = addon.tooltip
local core = addon.core
local minimap = addon.minimap
local debug = addon.debug
--local goals = addon.goals
local _G = _G

local cm

local realm = GetRealmName()

local function TimeBasedResets(menu, level)
   for _, v in pairs(util.TimeList) do
      menu:AddOption(v, level, function(self) util.ResetTime(v) menu:CloseMenu() end)
   end
end

local function TimeBasedAvgResets(menu, level)
   for _, v in pairs(util.TimeList) do
      if util.getSetValue(util.TimeSet, v) then
         menu:AddOption(format("Avg (%s)", v), level, function(self) util.ResetAvg(v) menu:CloseMenu() end)
      end
   end
end

-- Right-Click context cm initialization
local function ContextMenu_OnLoad(self, level, menuList)
   if cm then
      if level == 1 then
         -- If SexyMap is loaded this addon will allow SexyMap to control minimap icon movements and visibility
         -- Don't allow any context cm options to be visible that will not function
         local sexyMapLoaded = IsAddOnLoaded("SexyMap")

         cm:AddDescriptor    (  "WoW Money Statistics", 1, true )
         --[===[@debug
         -- TODO: Add Goals
         cm:AddOption        (  "Manage Goals",
                                    1,
                                    function(self)
                                       goals.ToggleDialog()
                                    end)
         cm:AddDescriptor    (  "-------------------", 1 )
         --@end-debug]===]
         cm:AddMenu          (  "Tooltip", 1, "Tooltips" )
         if not util.IsLDBInteraction() and (not sexyMapLoaded or not util.GetSexyCompatSetting()) then
            cm:AddMenu          (  "Minimap", 1, "Minimaps" )
         elseif util.IsLDBInteraction() then
            cm:AddCheckOption   (  "Hide Minimap Icon",
                                       function(self)
                                          return minimap.GetMinimapHideSetting()
                                       end,
                                       1,
                                       function(self, _, _, checked) 
                                          _G.WOWMMChar.Minimap.IsMinimapHidden = minimap.SetMinimapHidden(checked)
                                       end
                                    )
         end
         cm:AddDescriptor    (  "-------------------", 1 )
         cm:AddMenu          (  "Font", 1, "Fonts" )
         cm:AddMenu          (  "Font Size", 1, "FontSize" )
         cm:AddDescriptor    (  "-------------------", 1 )
         cm:AddMenu          (  "Remove Character", 1, "Removes" )
         cm:AddMenu          (  "Reset", 1, "Resets" )
         --[===[@debug
         cm:AddDescriptor    (  "-------------------", 1 )
         cm:AddMenu          (  "Debug", 1, "Debug" )
         --@end-debug]===]
      elseif menuList == "Tooltips" then
         tt.OptionalSettingMenu(cm, 2)
      elseif menuList == "Debug" then
         debug.debugContextMenu(cm, 2)
      elseif menuList == "ResetDay" then
         debug.WeeklyResetDay(cm, 3)
      elseif menuList == "Minimaps" then
         cm:AddCheckOption   (  "Lock to Minimap",
                                    function(self)
                                       return minimap.GetMinimapLockSetting()
                                    end,
                                    2,
                                    function(self, _, _, checked) 
                                       _G.WOWMMChar.Minimap.IsMinimapLocked = minimap.SetMinimapLock(checked)
                                    end
                                 )
         cm:AddCheckOption   (  "Lock",
                                    function(self)
                                       return minimap.GetLockSetting()
                                    end,
                                    2,
                                    function(self, _, _, checked) 
                                       _G.WOWMMChar.Minimap.IsLocked = minimap.SetLock(checked)
                                    end
                                 )
      elseif menuList == "Fonts" then
         local fonts = {}
         for n, v in pairs(_G["Prototype_Fonts"]) do
            if type(v) ~= "table" then 
               table.insert(fonts, n)
            end
         end
         table.sort(fonts)
         for _, v in pairs(fonts) do
            cm:AddRadioOption   (  v,
                                       function(self)
                                          return util.GetFontSetting() == v
                                       end,
                                       2,
                                       function(self, _, _, _)
                                          util.SetFontSetting(v)
                                          tt.UpdateFontInfo()
                                          cm:Refresh()
                                       end
                                    )
         end      
      elseif menuList == "FontSize" then
         local fontSizes = _G["Prototype_Fonts"].Sizes
         for _, v in pairs(fontSizes) do
            cm:AddRadioOption   (  tostring(v),
                                       function(self)
                                          return util.GetFontSizeSetting() == v
                                       end,
                                       2,
                                       function(self, _, _, _)
                                          util.SetFontSizeSetting(v)
                                          tt.UpdateFontInfo()
                                          cm:Refresh()
                                       end
                                    )
         end
      elseif menuList == "Removes" then
         for n, v in pairs(_G.WOWMMGlobal) do
            if type(v) == "table" then
               cm:AddMenu       (  n, 2, n )
            end
         end
      elseif menuList == "Resets" then
         TimeBasedResets(cm, 2);
         cm:AddDescriptor    (  "-------------------", 2 )
         TimeBasedAvgResets(cm, 2)
      else
         if level == 3 then
            local chars = util.GetSortedCharList(menuList)
            for _, cn in pairs(chars) do
               cm:AddOption  (  cn,
                                    3,
                                    function(self)
                                       util.RemoveCharacter(menuList, cn)
                                    end
                                 )
            end
         end
      end
   end
end

cm = Prototype_Menu:NewContextMenu  (addon_name.."ContextMenu",
                                       {
                                          OnLoad = ContextMenu_OnLoad,
                                       }
                                    )

function menu.IsMenuOpen()
   return cm and cm:IsMenuOpen()
end

function menu.ToggleMenu(anchor)
   if cm then
      cm:ToggleMenu(anchor)
   end
end

function menu.OpenMenu(anchor)
   if cm then
      cm:OpenMenu(anchor)
   end
end

function menu.CloseMenu()
   if cm then
      cm:CloseMenu()
   end
end

menu.GetDOWSetting = GetDOWSetting
